/* ************************************************************************
*   File: house.c                                       Part of CircleMUD *
*  Usage: Handling of player houses                                       *
*																		  *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
*                                                                         *
*  Updated by Templar Viper (templarviper@hotmail.com) in jan/feb 2003    *
************************************************************************ */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "comm.h"
#include "handler.h"
#include "db.h"
#include "interpreter.h"
#include "utils.h"
#include "house.h"
#include "genhouse.h"
#include "constants.h"
#include "diskio.h"

#define	LOC_ROOM			0
#define	MAX_BAG_ROWS		5

/* external functions */
bitvector_t asciiflag_conv(char *flag);
int Obj_to_store(struct obj_data *obj, FBFILE *fp, int locate, bool corpse);

/* external variables */
extern int mini_mud;
extern struct house_data *house_index;
extern int top_of_houset;

/* local functions */
int ok_house_room(house_vnum vnum, room_vnum room);
int	load_house_objects(house_vnum vnum);
int House_save(struct obj_data *obj, FBFILE *fp);
int Houseobj_get_filename(room_vnum vnum, char *fname, size_t maxlen);
int House_get_filename(house_vnum vnum, char *fname, size_t maxlen);
void house_restore_weight(struct obj_data *obj);
void house_delete_file(house_vnum vnum);
int	find_house(room_vnum vnum);
int	find_cowner(struct char_data *ch, int i);
int House_can_enter(struct char_data *ch, room_vnum house);
int house_can_take(struct char_data *ch, room_vnum house);
void hcontrol_list_houses(struct char_data *ch);
void hcontrol_build_house(struct char_data *ch, char *arg);
void hcontrol_pay_house(struct char_data *ch, char *arg);
void parse_house(FILE *fl, house_vnum vnum);
int	handle_house_obj(struct obj_data *temp, room_rnum rnum, int locate, struct obj_data **cont_row);
ACMD(do_hcontrol);
ACMD(do_house);

/******************************************************************
 *  Functions for house administration (creation, deletion, etc.  *
 *****************************************************************/

/* crash-save all the houses */
void House_save_all(void)
{
  save_houses();
}

void House_boot(void)
{
  FILE *fl, *housefile;
  char line[READ_SIZE], buf[64];
  int nr;

  if (!(fl = fopen(HCONTROL_FILE, "r"))) {
    mudlog(BRF, LVL_IMPL, TRUE, "SYSERR: Failed to open %s.", HCONTROL_FILE);
    return;
  } 
  
  for (;;) {
    if (!get_line(fl, line)) 
	  return;

    if (*line == '$')
      return;

    if (*line == '#') {
      if (sscanf(line, "#%d", &nr) != 1) {
	    log("SYSERR: Format error in %s.", HCONTROL_FILE);
	    exit(1);
	  }
      sprintf(buf, "%s%d.house", LIB_HOUSE, nr);

      if (!(housefile = fopen(buf, "r"))) {
        log("SYSERR: Failed to read %s.", buf);
        return;
	  } 
      
	  if (!house_index) 
		CREATE(house_index, struct house_data, top_of_houset + 1);
	  else 
		RECREATE(house_index, struct house_data, top_of_houset + 1);
	  
	  parse_house(housefile, nr);
	  fclose(housefile);
	  log("   House #%d loaded.", nr);
	}
  }

  fclose(fl);
}

int load_house_objects(room_vnum vnum) {
  FBFILE *fl;
  char fname[MAX_STRING_LENGTH], line[256];
  char f1[128], f2[128], f3[128], f4[128];
  int t[10], i, locate=0, nr, num_objs=0, rnum;
  struct obj_data *temp, *cont_row[MAX_BAG_ROWS];
 
  if ((rnum = real_room(vnum)) == NOWHERE) 
    return 1;

  if (!Houseobj_get_filename(vnum, fname, sizeof(fname)))
    return 1;
  
  for (i = 0; i < MAX_BAG_ROWS; i++)
    cont_row[i] = NULL;

  if (!(fl = fbopen(fname, FB_READ))) {
    if (errno != ENOENT) /* if it fails, NOT because of no file */
      log("SYSERR: Reading house object file. vnum=%d", vnum);
	else 
	  return 1;
  }

  temp = NULL;
  while (fbgetline(fl, line)) {
    char tag[6];
    int num;

    /* first, we get the number. Not too hard. */
    if(*line == '$' && line[1] == '~') {
      if (temp)
        num_objs += handle_house_obj(temp, rnum, locate, cont_row);
      break;
    }
    if (*line == '#') {
      if (sscanf(line, "#%d", &nr) != 1) {
        continue;
      } else {
        if (temp)
          num_objs += handle_house_obj(temp, rnum, locate, cont_row);
        temp = NULL;
        locate = 0;
      }
      /* we have the number, check it, load obj. */
      if (nr == NOTHING) {   /* then it is unique */
        temp = create_obj();
        temp->item_number=NOTHING;
      } else if (nr < 0) {
        continue;
      } else {
        if(nr >= 999999) 
          continue;
        if(real_object(nr) >= 0) {
          temp=read_object(nr,VIRTUAL);
          if (!temp) {
            continue;
          }
        } else {
          log("Nonexistent object %d found in house object file.", nr);
        }
      }
    }

    tag_argument(line, tag);
    num = atoi(line);

    switch(*tag) {
    case 'A':
      if (!strcmp(tag, "ADes"))
        temp->action_description = fbgetstring(fl);
      else if (!strcmp(tag, "Aff ")) {
        sscanf(line, "%d %d %d", &t[0], &t[1], &t[2]);
        if (t[0] < MAX_OBJ_AFFECT) {
          temp->affected[t[0]].location = t[1];
          temp->affected[t[0]].modifier = t[2];
        }
      }
      break;
    case 'C':
      if (!strcmp(tag, "Cost"))
        GET_OBJ_COST(temp) = num;
      break;
    case 'D':
      if (!strcmp(tag, "Desc"))
        temp->description = strdup(line);
      break;
    case 'E':
      if(!strcmp(tag, "EDes")) {
        struct extra_descr_data *new_desc;

        if (temp->item_number >= 0 && /* Regular object */
            temp->ex_description &&   /* with ex_desc == prototype */
            (temp->ex_description == obj_proto[real_object(temp->item_number)].ex_description))
          temp->ex_description = NULL;
        CREATE(new_desc, struct extra_descr_data, 1);
        new_desc->keyword = fbgetstring(fl);
        new_desc->description = fbgetstring(fl);
        new_desc->next = temp->ex_description;
        temp->ex_description = new_desc;
      }
      break;
    case 'F':
      if (!strcmp(tag, "Flag")) {
		sscanf(line, "%s %s %s %s", f1, f2, f3, f4);
        GET_OBJ_EXTRA(temp)[0] = asciiflag_conv(f1);
        GET_OBJ_EXTRA(temp)[1] = asciiflag_conv(f2);
        GET_OBJ_EXTRA(temp)[2] = asciiflag_conv(f3);
        GET_OBJ_EXTRA(temp)[3] = asciiflag_conv(f4);
	  }
	  break;
    case 'L':
      if(!strcmp(tag, "Loc "))
        locate = num;
      break;
    case 'N':
      if (!strcmp(tag, "Name"))
        temp->name = strdup(line);
      break;
    case 'P':
	  if (!strcmp(tag, "Perm")) {
		sscanf(line, "%s %s %s %s", f1, f2, f3, f4);
        GET_OBJ_PERM(temp)[0] = asciiflag_conv(f1);
        GET_OBJ_PERM(temp)[1] = asciiflag_conv(f2);
        GET_OBJ_PERM(temp)[2] = asciiflag_conv(f3);
        GET_OBJ_PERM(temp)[3] = asciiflag_conv(f4);
	  }
      break;
    case 'R':
      if (!strcmp(tag, "Rent"))
        GET_OBJ_RENT(temp) = num;
      break;
    case 'S':
      if (!strcmp(tag, "Shrt"))
        temp->short_description = strdup(line);
      break;
    case 'T':
      if (!strcmp(tag, "Type"))
        GET_OBJ_TYPE(temp) = num;
      break;
    case 'W':
	  if (!strcmp(tag, "Wear")) {
		sscanf(line, "%s %s %s %s", f1, f2, f3, f4);
        GET_OBJ_WEAR(temp)[0] = asciiflag_conv(f1);
        GET_OBJ_WEAR(temp)[1] = asciiflag_conv(f2);
        GET_OBJ_WEAR(temp)[2] = asciiflag_conv(f3);
        GET_OBJ_WEAR(temp)[3] = asciiflag_conv(f4);
	  } else if (!strcmp(tag, "Wght"))
        GET_OBJ_WEIGHT(temp) = num;
      break;
    case 'V':
      if (!strcmp(tag, "Vals")) {
        sscanf(line, "%d %d %d %d", &t[0], &t[1], &t[2], &t[3]);
        for (i = 0; i < 4; i++)
          GET_OBJ_VAL(temp, i) = t[i];
      }
      break;
    }
  }
 
  fbclose(fl);

  return 0;
}

/* Save all objects for a house (recursive; initial call must be followed
   by a call to House_restore_weight) */
int House_save(struct obj_data *obj, FBFILE *fp)
{
  struct obj_data *tmp;
  int result;
 
  if (obj) {
    House_save(obj->contains, fp);
    House_save(obj->next_content, fp);
    result = Obj_to_store(obj, fp, 0, FALSE);
    if (!result) 
      return (0);
	

    for (tmp = obj->in_obj; tmp; tmp = tmp->in_obj)
      GET_OBJ_WEIGHT(tmp) -= GET_OBJ_WEIGHT(obj);
  }
  
  return (1);
}

/* Save all objects in a house (recurses through all rooms) */
void house_crashsave_objects(house_vnum vnum)
{
  FBFILE *fp;
  house_rnum hrnum;
  room_rnum rnum;
  int index;
  char buf[MAX_STRING_LENGTH];

  if ((hrnum = real_house(vnum)) == NOWHERE)
	return;

  /* Save down each room in the house separately. */
  for (index = 0; HOUSE_ROOM(hrnum, index) != NOWHERE; index++) {
	if ((rnum = real_room(HOUSE_ROOM(hrnum, index))) == NOWHERE)
      continue;
	if (!ROOM_FLAGGED(rnum, ROOM_HOUSE_CRASH))
	  continue;
	
	/* Create the house object file */
    if (!Houseobj_get_filename(vnum, buf, sizeof(buf)))
      continue;

    if (!(fp = fbopen(buf, FB_WRITE))) 
      log("SYSERR: Writing house object file. vnum=%d", vnum);

    if (!House_save(world[rnum].contents, fp))
      continue;
    
	fbprintf(fp, "$~\n");
	fbclose(fp);

	house_restore_weight(world[rnum].contents);
	REMOVE_BIT_AR(ROOM_FLAGS(rnum), ROOM_HOUSE_CRASH);
  }
}

/* 
 * Called from house_auto_prune(). This function deletes
 * the files from disk, but leaves the house in memory.
 */
void house_delete_file(house_vnum vnum)
{
  char filename[50];
  FILE *fl;

  if (vnum == NOWHERE) {
	log("SYSERR: Invalid house_vnum passed to house_delete_file().");
    return;
  }
  
  if (!House_get_filename(vnum, filename, sizeof(filename)))
    return;
  if (!(fl = fopen(filename, "rb"))) {
    if (errno != ENOENT)	/* if it fails but NOT because of no file */
      log("SYSERR: Deleting house file %s: %s", filename, strerror(errno));
    return;
  }
  fclose(fl);

  /* if it fails, NOT because of no file */
  if (remove(filename) < 0 && errno != ENOENT)
    log("SYSERR: deleting house file %s: %s", filename, strerror(errno));

  return;
}

void check_empty_line(FILE *fl, char line[READ_SIZE], house_vnum vnum)
{
  if (!get_line(fl, line)) {
    log("SYSERR: Expecting data, but house file ended! (House #%d)", vnum);
    exit(1);
  }
}

void parse_house(FILE *fl, house_vnum vnum)
{
  static int house_nr = 0;
  room_rnum rhouse = NOWHERE, ratrium = NOWHERE;
  int count = 0, t[10], retval, j;
  long l[10];
  char line[READ_SIZE];
  
  HOUSE_NUM(house_nr) = vnum;

  check_empty_line(fl, line, vnum);
  
  /* Ignored */
  if (sscanf(line, " #%d", t) != 1) {
	log("SYSERR: Format error in vnum of house #%d", vnum);
	exit(1);
  }

  check_empty_line(fl, line, vnum);
   
  if (sscanf(line, " %d %d %ld %ld %ld %ld %d %d %d %d %d", 
	  t, t + 1, l, l + 1, l + 2, l + 3, t + 2, t + 3, t + 4, t + 5, t + 6) != 11) {
    log("SYSERR: Format error in house #%d", vnum);
    exit(1);
  }

  HOUSE_ATRIUM(house_nr)	 = t[0];
  HOUSE_EXIT(house_nr)	 	 = t[1];
  HOUSE_OWNER(house_nr)		 = l[0];
  HOUSE_BUILT(house_nr)		 = l[1];
  HOUSE_USED(house_nr)		 = l[2];
  HOUSE_PAID(house_nr)		 = l[3];
  HOUSE_PRUNE_SAFE(house_nr) = t[2];
  HOUSE_MODE(house_nr)       = t[3];
  HOUSE_COST(house_nr)       = t[4];
  HOUSE_MAX_SECURE(house_nr) = t[5];
  HOUSE_MAX_LOCKED(house_nr) = t[6];

  /* checking */
  if (get_name_by_id(HOUSE_OWNER(house_nr)) == NULL)
	log("House %d has an invalid owner.", HOUSE_NUM(house_nr));
  
  if ((ratrium = real_room(HOUSE_ATRIUM(house_nr))) == NOWHERE) {
	log("House %d not loaded because house atrium does not exist.", HOUSE_NUM(house_nr));
	return;	/* house doesn't have an atrium */
  }
  
  if (HOUSE_EXIT(house_nr) < 0 || HOUSE_EXIT(house_nr) >= NUM_OF_DIRS) {
	log("House %d not loaded due to an invalid exit number.", HOUSE_NUM(house_nr));
	return;	/* invalid exit num */
  }

  check_empty_line(fl, line, vnum);

  /* Only 10 rooms max. Hm. */
  if ((retval = sscanf(line, " %d %d %d %d %d %d %d %d %d %d", t, t + 1, t + 2, t + 3, t + 4,
	  t + 5, t + 6, t + 7, t + 8, t + 9)) == 0) {
    log("SYSERR: Format error in room section of house #%d", vnum);
    return;
  }
  
  CREATE(house_index[house_nr].rooms, long, retval + 1);
  for (j = 0; j != retval; j++)
    HOUSE_ROOM(house_nr, j) = t[j]; 
  HOUSE_ROOM(house_nr, j) = NOWHERE;

  check_empty_line(fl, line, vnum);

  /* Owner list */
  if ((retval = sscanf(line, " %d %d %d %d %d %d %d %d %d %d", t, t + 1, t + 2, t + 3, t + 4,
	  t + 5, t + 6, t + 7, t + 8, t + 9)) == 0) {
    log("SYSERR: Format error in owner section of house #%d", vnum);
    return;
  }
  
  CREATE(house_index[house_nr].cowners, long, retval + 1);
  for (j = 0; j != retval; j++)
    HOUSE_COWNER(house_nr, j) = t[j]; 
  HOUSE_COWNER(house_nr, j) = NOBODY;
  
  check_empty_line(fl, line, vnum);

  /* Guest list */
  if ((retval = sscanf(line, " %d %d %d %d %d %d %d %d %d %d", t, t + 1, t + 2, t + 3, t + 4,
	  t + 5, t + 6, t + 7, t + 8, t + 9)) == 0) {
    log("SYSERR: Format error in guest section of house #%d", vnum);
    return;
  }

  CREATE(house_index[house_nr].guests, long, retval + 1);
  for (j = 0; j != retval; j++)
    HOUSE_GUEST(house_nr, j) = t[j]; 
  HOUSE_GUEST(house_nr, j) = NOBODY;

  if ((rhouse = real_room(HOUSE_ROOM(house_nr, 0))) == NOWHERE) {
	log("House %d not loaded because house room does not exist.", HOUSE_NUM(house_nr));
	return;	/* this vnum doesn't exist -- skip */
  }

  if (TOROOM(rhouse, HOUSE_EXIT(house_nr)) != ratrium) {
	log("House %d not loaded because exit #%d did not lead to atrium #%d. (rhouse=%d)", 
		HOUSE_NUM(house_nr), HOUSE_EXIT(house_nr), ratrium, rhouse);
	return;	/* exit num mismatch -- skip */
  }

  SET_BIT_AR(ROOM_FLAGS(ratrium), ROOM_ATRIUM);

  for (count = 0; HOUSE_ROOM(house_nr, count) != NOWHERE; count++) {
	if ((rhouse = real_room(HOUSE_ROOM(house_nr, count))) == NOWHERE) {
	  log("Room %ld skipped because it does not exist.", HOUSE_ROOM(house_nr, count));
	  continue;	/* this vnum doesn't exist -- skip */
	}
	SET_BIT_AR(ROOM_FLAGS(rhouse), ROOM_HOUSE);
	SET_BIT_AR(ROOM_FLAGS(rhouse), ROOM_PRIVATE);
	load_house_objects(HOUSE_ROOM(house_nr, count));
  }
  
  top_of_houset = house_nr++;
}


/******************************************************************
 *  "House control" functions                                     *
 *****************************************************************/

const char *HCONTROL_FORMAT =
"Usage: hcontrol pay <house vnum>\r\n"
"       hcontrol show\r\n";

void hcontrol_pay_house(struct char_data *ch, char *arg)
{
  int i;

  if (!*arg)
    send_to_char(ch, "%s", HCONTROL_FORMAT);
  else if ((i = find_house(atoi(arg))) == NOWHERE)
    send_to_char(ch, "Unknown house.\r\n");
  else {
    mudlog(NRM, MAX(LVL_IMMORT, GET_INVIS_LEV(ch)), TRUE, "Payment for house %s collected by %s.", arg, GET_NAME(ch));

    HOUSE_PAID(i) = time(0);
    send_to_char(ch, "Payment recorded.\r\n");
  }
}

void hcontrol_list_houses(struct char_data *ch)
{
  int i, guests;
  char *timestr, *temp;
  char built_on[128], last_pay[128], last_used[128], own_name[MAX_NAME_LENGTH + 1];

  if (!house_index) {
    send_to_char(ch, "No houses have been build yet.\r\n");
    return;
  }
  send_to_char(ch,
	"Address  Atrium  Build Date  Guests  Owner        Last Paymt Last Used\r\n"
	"-------  ------  ----------  ------  ------------ ---------- ---------\r\n");

  for (i = 0; i <= top_of_houset; i++) {
    /* Avoid seeing <UNDEF> entries from self-deleted people. -gg 6/21/98 */
    if ((temp = get_name_by_id(HOUSE_OWNER(i))) == NULL)
	  continue;

    if (HOUSE_BUILT(i)) {
      timestr = asctime(localtime(&(HOUSE_BUILT(i))));
      *(timestr + 10) = '\0';
      strlcpy(built_on, timestr, sizeof(built_on));
    } else
      strcpy(built_on, "Unknown");	/* strcpy: OK (for 'strlen("Unknown") < 128') */

    if (HOUSE_PAID(i)) {
      timestr = asctime(localtime(&(HOUSE_PAID(i))));
      *(timestr + 10) = '\0';
      strlcpy(last_pay, timestr, sizeof(last_pay));
    } else
      strcpy(last_pay, "None");	/* strcpy: OK (for 'strlen("None") < 128') */

    if (HOUSE_USED(i)) {
	  timestr = asctime(localtime(&(HOUSE_USED(i))));
	  *(timestr + 10) = '\0';
	  strlcpy(last_pay, timestr, sizeof(last_used));
	} else
	  strcpy(last_pay, "None");	/* strcpy: OK (for 'strlen("None") < 128') */

    for (guests = 0; HOUSE_GUEST(i, guests) != NOBODY; guests++); /* Loop does the trick */
	
	/* Now we need a copy of the owner's name to capitalize. -gg 6/21/98 */
    strcpy(own_name, temp);	/* strcpy: OK (names guaranteed <= MAX_NAME_LENGTH+1) */
    send_to_char(ch, "%7d %7d  %-10s    %2d    %-12s %-10s %s\r\n",
	    HOUSE_NUM(i), HOUSE_ATRIUM(i), built_on,
	    guests, CAP(own_name), last_pay, last_used);
  }
}

/* The hcontrol command itself, used by imms to pay/show houses */
ACMD(do_hcontrol)
{
  char arg1[MAX_INPUT_LENGTH], arg2[MAX_INPUT_LENGTH];

  half_chop(argument, arg1, arg2);

  if (is_abbrev(arg1, "pay"))
    hcontrol_pay_house(ch, arg2);
  else if (is_abbrev(arg1, "show"))
    hcontrol_list_houses(ch);
  else
    send_to_char(ch, "%s", HCONTROL_FORMAT);
}

/* 
 * The house command, used by mortal house owners to assign guests 
 * and cowners. 
 */
ACMD(do_house)
{
  char arg1[MAX_INPUT_LENGTH], arg2[MAX_INPUT_LENGTH];
  int i, j, id;

  half_chop(argument, arg1, arg2);

  if (is_abbrev(arg1, "guests")) {
  	if (!ROOM_FLAGGED(IN_ROOM(ch), ROOM_HOUSE))
	  send_to_char(ch, "You must be in your house to set guests.\r\n");
	else if ((i = find_house(GET_ROOM_VNUM(IN_ROOM(ch)))) == NOWHERE)
	  send_to_char(ch, "Um.. this house seems to be screwed up.\r\n");
	else if (GET_IDNUM(ch) != HOUSE_OWNER(i) && !find_cowner(ch, i))
	  send_to_char(ch, "Only owners can set guests.\r\n");
	else if (!*arg2)
	  House_list_guests(ch, i, FALSE);
	else if ((id = get_id_by_name(arg2)) < 0)
	  send_to_char(ch, "No such player.\r\n");
	else if (id == GET_IDNUM(ch))
	  send_to_char(ch, "It's your house!\r\n");
	else {
	  for (j = 0; HOUSE_GUEST(i, j) != NOWHERE; j++)
		if (HOUSE_GUEST(i, j) == id) {
		  remove_from_house_list(&(HOUSE_GUESTS(i)), j);
		  send_to_char(ch, "Guest deleted.\r\n");
		  save_houses();
		  return;
		}
		if (j == HOUSE_MAX_GUESTS) {
		  send_to_char(ch, "You have too many guests.\r\n");
		  return;
		}
		add_to_house_list(&(HOUSE_GUESTS(i)), id);
		send_to_char(ch, "Guest added.\r\n");
		save_houses();
	}
  } else if (is_abbrev(arg1, "cowners")) {
	if (!ROOM_FLAGGED(IN_ROOM(ch), ROOM_HOUSE))
      send_to_char(ch, "You must be in your house to set co-owners.\r\n");
	else if ((i = find_house(GET_ROOM_VNUM(IN_ROOM(ch)))) == NOWHERE)
	  send_to_char(ch, "Um.. this house seems to be screwed up.\r\n");
	else if (GET_IDNUM(ch) != HOUSE_OWNER(i))
	  send_to_char(ch, "Only the primary owner can set co-owners.\r\n");
	else if (!*arg2)
	  House_list_cowners(ch, i, FALSE);
	else if ((id = get_id_by_name(arg2)) < 0)
	  send_to_char(ch, "No such player.\r\n");
	else if (id == GET_IDNUM(ch))
	  send_to_char(ch, "It's your house!\r\n");
	else {
	  for (j = 0; HOUSE_COWNER(i, j) != NOWHERE; j++)
		if (HOUSE_COWNER(i, j) == id) {
		  remove_from_house_list(&(HOUSE_COWNERS(i)), j);
		  send_to_char(ch, "Guest deleted.\r\n");
		  save_houses();
		  return;
		}
		if (j == HOUSE_MAX_COWNERS) {
		  send_to_char(ch, "You have too many co-owners.\r\n");
		  return;
		}
		add_to_house_list(&(HOUSE_COWNERS(i)), id);
		send_to_char(ch, "Co-owner added.\r\n");
		save_houses();
	}
  } else 
	send_to_char(ch, "Usage: %s { guests | cowners } <name>\r\n", CMD_NAME);
}

void House_list_guests(struct char_data *ch, int i, int quiet)
{
  int j;
  char *temp;

  for (j = 0; HOUSE_GUEST(i, j) != NOBODY; j++);
  if (!j && !quiet) {
    send_to_char(ch, "  Guests: None\r\n");
    return;
  }

  send_to_char(ch, "  Guests: ");

  for (j = 0; HOUSE_GUEST(i, j) != NOBODY; j++) {
	if ((temp = get_name_by_id(HOUSE_GUEST(i, j))) == NULL)
	  continue;

	send_to_char(ch, "%c%s ", UPPER(*temp), temp + 1);
  }

  send_to_char(ch, "\r\n");
}

void House_list_cowners(struct char_data *ch, int i, int quiet)
{
  int j;
  char *temp;

  for (j = 0; HOUSE_GUEST(i, j) != NOBODY; j++);
  if (!j && !quiet) {
    send_to_char(ch, "  Guests: None\r\n");
    return;
  }

  send_to_char(ch, "  Co-owners: ");

  for (j = 0; HOUSE_COWNER(i, j) != NOBODY; j++) {
	if ((temp = get_name_by_id(HOUSE_COWNER(i, j))) == NULL)
	  continue;

	send_to_char(ch, "%c%s ", UPPER(*temp), temp + 1);
  }

  send_to_char(ch, "\r\n");
}


/******************************************************************
 *  Misc. functions                                               *
 *****************************************************************/

int	find_house(room_vnum vnum)
{
  int i;
  
  if (!house_index)
	return (NOWHERE);

  for (i = 0; i <= top_of_houset; i++)
	if (ok_house_room(i, vnum))
	  return (i);

  return (NOWHERE);
}

int	find_house_owner(struct char_data *ch)
{
  int i;

  if (!house_index)
	return (NOWHERE);

  for (i = 0; i <= top_of_houset; i++)
	if (HOUSE_OWNER(i) == GET_IDNUM(ch))
      return (HOUSE_NUM(i));

  return (NOWHERE);
}

int	find_cowner(struct char_data *ch, int i)
{
  int j;

  if (!house_index)
	return (0);

  for (j = 0; HOUSE_COWNER(i, j) != NOWHERE; j++)
	if (HOUSE_COWNER(i, j) == GET_IDNUM(ch))
	  return (1);
  
  return (0);
}

int	find_guest(struct char_data *ch, int i)
{
  int j;

  if (!house_index)
	return (0);

  for (j = 0; HOUSE_GUEST(i, j) != NOWHERE; j++)
	if (HOUSE_GUEST(i, j) == GET_IDNUM(ch))
	  return (1);

  return (0);
}

int ok_house_room(house_rnum rnum, room_vnum room)
{
  int index;

  if (!house_index)
 	return (NOWHERE);

  for (index = 0; HOUSE_ROOM(rnum, index) != NOWHERE; index++)
	if (HOUSE_ROOM(rnum, index) == room)
      return (TRUE);
  
  return (FALSE);
}

/* note: arg passed must be house vnum, so there. */
int House_can_enter(struct char_data *ch, room_vnum house)
{
  int i;

  if (GET_LEVEL(ch) >= LVL_GRGOD || (i = find_house(house)) == NOWHERE)
    return (1);

  switch (house_index[i].mode) {
  case HOUSE_PRIVATE:
    if (GET_IDNUM(ch) == HOUSE_OWNER(i)) {
	  HOUSE_USED(i) = time(0);
      return (1);
	}
    if (find_cowner(ch, i)) {
	  HOUSE_USED(i) = time(0);
	  return (1);
	}
	if (find_guest(ch, i))
	  return (1);
	
	break;
  }

  return (0);
}

int house_can_take(struct char_data *ch, room_vnum room)
{
  int i;

  if (GET_LEVEL(ch) >= LVL_GRGOD || (i = find_house(room)) == NOWHERE)
	return (1);

  switch (house_index[i].mode) {
	case HOUSE_PRIVATE:
	  if (GET_IDNUM(ch) == HOUSE_OWNER(i))
		return (1);
	  if (find_cowner(ch, i))
	 	return (1);
	  break;
  }

  return (0);
}

void house_auto_prune(void)
{
  int i, j;
  room_rnum real_atrium, real_house;

  for (i = 0; i < top_of_houset; i++) {
	if ((real_house = real_room(house_index[i].vnum)) != NOWHERE) {
      if (!house_index[i].prune_safe && (time(0) - house_index[i].last_used > 14 * SECS_PER_REAL_DAY)) {

		if ((real_atrium = real_room(house_index[i].atrium)) == NOWHERE)
          log("House %d had invalid atrium %d!", real_house, house_index[i].atrium);
		else
		  REMOVE_BIT_AR(ROOM_FLAGS(real_atrium), ROOM_ATRIUM);
			
		if ((real_house = real_room(house_index[i].vnum)) == NOWHERE)
		  log("House %d had invalid vnum %d!", real_house, house_index[i].vnum);
		else {
		  REMOVE_BIT_AR(ROOM_FLAGS(real_house), ROOM_HOUSE);
		  REMOVE_BIT_AR(ROOM_FLAGS(real_house), ROOM_PRIVATE);
		  REMOVE_BIT_AR(ROOM_FLAGS(real_house), ROOM_HOUSE_CRASH);
		}
			
		house_delete_file(house_index[i].vnum);

		mudlog(NRM, LVL_IMMORT, TRUE, "House %d auto-pruned.", house_index[i].vnum);
		
		for (j = i; j < top_of_houset - 1; j++)
		  house_index[j] = house_index[j + 1];
			
		top_of_houset--;
			
		save_houses();
	
		/*
		 * Now, reset the ROOM_ATRIUM flag on all existing houses' atriums,
		 * just in case the house we just deleted shared an atrium with another
		 * house.  --JE 9/19/94
		 */
		for (i = 0; i < top_of_houset; i++)
		  if ((real_atrium = real_room(house_index[i].atrium)) != NOWHERE)
			SET_BIT_AR(ROOM_FLAGS(real_atrium), ROOM_ATRIUM);
	  }
	}
  }
}

int House_get_filename(house_vnum vnum, char *fname, size_t maxlen)
{
  if (vnum == NOWHERE)
    return (0);

  snprintf(fname, maxlen, LIB_HOUSE"%d.house", vnum);
  return (1);
}

/* Return a filename given a house vnum */
int Houseobj_get_filename(room_vnum vnum, char *fname, size_t maxlen)
{
  if (vnum == NOWHERE)
    return (0);

  snprintf(fname, maxlen, LIB_HOUSEOBJ"%d.obj", vnum);
  return (1);
}

/* This function is also used for corpses */
int	handle_house_obj(struct obj_data *temp, room_rnum rnum, int locate, struct obj_data **cont_row)
{
  int j;
  struct obj_data *obj1;

  if (!temp)
	return (0);
  
  obj_to_room(temp, rnum);

  for (j = MAX_BAG_ROWS-1;j > -locate;j--)
	if (cont_row[j]) { /* no container -> back to room */
      for (;cont_row[j];cont_row[j] = obj1) {
		obj1 = cont_row[j]->next_content;
		obj_to_room(cont_row[j], rnum);
	  } 
	  cont_row[j] = NULL;
	}

  if (j == -locate && cont_row[j]) { /* content list existing */
	if (GET_OBJ_TYPE(temp) == ITEM_CONTAINER) {
      /* take item ; fill ; put in room again */
	  obj_from_room(temp);
	  temp->contains = NULL;
	  for (;cont_row[j];cont_row[j] = obj1) {
	 	obj1 = cont_row[j]->next_content;
		obj_to_obj(cont_row[j], temp);
	  }
	  obj_to_room(temp, rnum); /* add to room first ... */
	} else { /* object isn't container -> empty content list */
	  for (;cont_row[j];cont_row[j] = obj1) {
		obj1 = cont_row[j]->next_content;
		obj_to_room(cont_row[j], rnum);
	  }
	  cont_row[j] = NULL;
	}
  }

  if (locate < 0 && locate >= -MAX_BAG_ROWS) {
	/*
	 * let obj be part of content list
	 * but put it at the list's end thus having the items
	 * in the same order as before renting
	 */
	obj_from_room(temp);
	if ((obj1 = cont_row[-locate-1])) {
      while (obj1->next_content)
		obj1 = obj1->next_content;
	  obj1->next_content = temp;
	} else
	  cont_row[-locate-1] = temp;
  }
  return (1);
}

/* restore weight of containers after house_save has changed them for saving */
void house_restore_weight(struct obj_data *obj)
{
  if (obj) {
	house_restore_weight(obj->contains);
	house_restore_weight(obj->next_content);
	if (obj->in_obj)
      GET_OBJ_WEIGHT(obj->in_obj) += GET_OBJ_WEIGHT(obj);
  }
}

